/*
fix for v12.88 running status bug
-when programs on 'other' muxes finish, the epg is not always updated properly.
-this results in programs that have already finished being displayed on the
'info' button and as the current program in the epg  The epg colours are also
incorrect.

The cause seems to be a failure to always correctly update the running status.

---
What seems to happen is that when a program on another mux finishes, the
firmware fails to update the running status in the EPG properly.  This results
in the Info box showing the already finished program or the EPG starting with
the wrong program, and getting the colours all wrong.

Internally, multiple programs have the running status set to 'running' (or
'now'), which confuses the EPG display routines. Once the next program finishes,
the problem can get worse, the 2nd/3rd entry may go 'grey', and the EPG gets
'stuck' on the earlier program,  The problem eventually clears itself, as the
firmware will automatically delete entries once they have been finished for more
than about an hour, but at times it can appear as if the EPG is running about an
hour slow. Sometimes the EPG will correct itself after switching to the affected
channel, but often not.

The problem is at its worst if you refrain from channel hopping - i.e. tune to
bbc1; wait ~ 4 hours; now examine the epg - most of the other channels will be
in chaos.

To replicate:
-remove all taps from autostart
-tune to bbc1
-reboot
-wait a few minutes for the epg to populate
-without changing channels, wait for a program on another mux (e.g.
itv/ch4/five) to end (i.e. typically across a half hour boundary)
-without changing channels, examine the EPG for these channels

It is likely that the EPG will now show 2 programs set to 'runnning' (slightly
lighter blue background). If you now change to an affected channel, the Info
dialog will stick on the earlier program.

e.g.

@23:30 - ok. Love Island is running, and will appear at top of EPG list, and in Info box
[19d8:27e7:336f] [233a:2005:2045:c296] [19/07/2006, 22:30-23:00] [2] [       ] [2 0] ITV News
[05e8:1e87:14d2] [233a:2005:2045:c297] [19/07/2006, 23:00-23:35] [2] [  now  ] [3 0] Love Island
[02b4:1e88:10d9] [233a:2005:2045:c298] [19/07/2006, 23:35-00:00] [2] [  next ] [2 0] Club Reps
[1813:27e8:10ff] [233a:2005:2045:c299] [20/07/2006, 00:00-03:00] [2] [       ] [3 0] ITV Play: The Mint
[1b16:1dc5:2e19] [233a:2005:2045:c182] [20/07/2006, 03:00-04:05] [2] [       ] [4 0] British Touring Cars...

@23:40 - broken, Club Reps is running, but Love Island and will appear at top of EPG list, and in Info box
[19d8:27e7:336f] [233a:2005:2045:c296] [19/07/2006, 22:30-23:00] [2] [       ] [2 0] ITV News
[05e8:1e87:14d2] [233a:2005:2045:c297] [19/07/2006, 23:00-23:35] [2] [  now  ] [3 0] Love Island
[02b4:1e88:10d9] [233a:2005:2045:c298] [19/07/2006, 23:35-00:00] [2] [  now  ] [2 0] Club Reps
[1813:27e8:10ff] [233a:2005:2045:c299] [20/07/2006, 00:00-03:00] [2] [  next ] [3 0] ITV Play: The Mint
[1b16:1dc5:2e19] [233a:2005:2045:c182] [20/07/2006, 03:00-04:05] [2] [       ] [4 0] British Touring Cars...

*/

typedef unsigned char  UCHAR;
typedef unsigned short USHORT;
typedef unsigned long  ULONG;
#include <tap.h>
#include "exTap.h"
#include "tf5800.h"
#include "hash.h"

//Every TAP application needs unique ID and TAP allocations that have same ID cant execute at same time.
//ID consists of DWORD( 4byte ) and the MSB bit should be set by 1.
//The following is how to use in source codes.
#define ID_USERTAP 0xbdb01001
TAP_ID( ID_USERTAP );

//Every TAP application has to include Program name, Author name, and Program description as following
//manners in source code.
TAP_PROGRAM_NAME("running_status_fix v0.01");
TAP_AUTHOR_NAME("bdb");
TAP_DESCRIPTION("fix for v12.88 running status bug");
TAP_ETCINFO(__DATE__);

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
#include "TSRCommander.inc"

extern void TSRCommanderConfigDialog();
extern bool TSRCommanderExitTAP();
extern int is_tsr_commander_running(void);

//----------------------------------------------------------------------------
//
//main
//
extern type_menu main_menu;
extern type_menu more_menu;

extern int TAP_Main(void);
extern dword TAP_EventHandler(word event, dword param1, dword param2);
extern int main_menu_handler(type_menu *menu, dword param1);
extern int more_menu_handler(type_menu *menu, dword param1);
extern int key_handler(dword param1);

extern int init(void);
extern int close(void);

int thirty_seconds = 0;
int debug_enabled = 0;

void *service_hash = 0;
int init_service_id_filter();
int close_service_id_filter();

void dump_event(type_eventtable *fw_et, int event_index);
int dump_now_next(int service_id);
int fix_running_status();

//----------------------------------------------------------------------------
//
//tsr_commander stuff
//
bool TSRCommanderExitTAP()
{
    close();
    TAP_Exit();
    return FALSE;
}

void TSRCommanderConfigDialog()
{
    ShowMenu(&main_menu);
}

int is_tsr_commander_running(void)
{
    int i;
    type_TAPProcess *TAPProcess;

    for (i=0; i<16; i++) {
        TAPProcess = (type_TAPProcess *)(fw_tap_table + (i*32));
        if (TAPProcess->firmwaregp) {
            if (TAPProcess->header) {
                if (TAPProcess->header->id == 0x807a10b2) {  //tsr_commander
                    return 1;
                }
            }
        }
    }
    return 0;
}

//----------------------------------------------------------------------------
//
//main
//
char *tap_name;
type_menu main_menu = {
    0,
    0,
    {0},
    0,
    (char *)__tap_program_name__,
    {
        {6, "xxx",              },
        {0, "Enable Running Status Fix",  },
        {1, "Debug ON",         },
        {2, " -",               },              //Fix Running Status
        {3, " -",               },              //Dump Running Status
        {4, "Exit Tap",         },
        {5, "More ...",         },
    },
};
type_menu more_menu = {
    0,
    0,
    {0},
   0,
    (char *)__tap_program_name__,
    {
        {8, "xxx",                  },

        {0, "Show Num Events",      },
        {1, "Delete all Events",    },
        {2, "Show Tuner",           },
        {3, "Show Current Genre",   },
        {4, " -",                   },
        {5, " -",                   },
        {6, "time info",            },
        {7, "memory info",          },
    },
};


int TAP_Main(void)
{
    int ret;
    word SysID = *(word*)pSysID;

    tap_name =  (char *)__tap_program_name__;
    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SERIAL, strLog, DELAY_SHORT);

    //determine support for different firmwares
    ret = exTAP_Init();
    if (ret) {
        sprintf(strLog, "Unsupported platform - %d:%04x", *(word*)pSysID, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_MEDIUM);
        return 0;
    }
    if ((fw_sysid==456)&&(_appl_version==0x1288)) {
    }
    else {
        sprintf(strLog, "Unsupported platform - %d:%04x", *(word*)pSysID, _appl_version);
        log_message(LOG_SCREEN_WAIT|LOG_SERIAL, strLog, DELAY_MEDIUM);
        return 0;
    }

    sprintf(strLog, "Opening %s", tap_name);
    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);

    init();
    return 1;                 //0=application mode, 1=tsr mode
}

dword TAP_EventHandler(word event, dword param1, dword param2)
{
    ULONG state, subState;
    int ret;

    switch (event) {
        case EVT_IDLE:
//          TAP_Print("\nEVT_IDLE, event=%x, param1=%x, param2=%x", event, param1, param2);
            if (message_showing) {
                if (TAP_GetTick() > message_showing) {
                    TAP_Osd_Delete(message_rgn);    // release rgn-handle
                    message_showing = 0;
                    message_rgn = 0;
                }
            }
            if (thirty_seconds) {
                if (TAP_GetTick() > thirty_seconds) {
//TAP_Print("\nthirty seconds:");
                    fix_running_status();
                    thirty_seconds = TAP_GetTick() + (30*100);
                }
            }
            TSRCommanderWork();
            break;

        case EVT_KEY:
//          TAP_Print("\nEVT_KEY, event=%x, param1=%x, param2=%x", event, param1, param2);
            TAP_GetState(&state, &subState);
            if ((state == STATE_Normal) && (subState == SUBSTATE_Normal)) {
                if (main_menu.visible) { // handling keys when options window is shown
                    param1 = main_menu_handler(&main_menu, param1);
                }
                else
                if (more_menu.visible) {
                    param1 = more_menu_handler(&more_menu, param1);
                }
                else {
                    param1 = key_handler(param1);
                }
            }
            break;
        case EVT_SVCSTATUS:
            break;
        default:
//          TAP_Print("\nEVT_???, event=%x, param1=%x, param2=%x", event, param1, param2);
            break;
    }
    return param1;
}

int main_menu_handler(type_menu *menu, dword param1)
{
    int selection;
    dword gp;
    int ret;
    int channel_type, channel_number;
    TYPE_TapChInfo channel_info;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            switch (selection) {
                case 0:
                    if (thirty_seconds) {
                        sprintf(strLog, "Disabling Running Status Fix");
                        strcpy(menu->item[1].str, "Enable Running Status Fix");
                        thirty_seconds = 0;
                    }
                    else {
                        sprintf(strLog, "Enabling Running Status Fix");
                        strcpy(menu->item[1].str, "Disable Running Status Fix");
                        //recheck every 30 seconds
                        thirty_seconds = TAP_GetTick() + (30*100);
                    }
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    return 0;
                case 1:
                    if (debug_enabled) {
                        sprintf(strLog, "Debug Disabled");
                        strcpy(menu->item[2].str, "Debug ON");
                        strcpy(menu->item[3].str, " -");
                        strcpy(menu->item[4].str, " -");
                        debug_enabled = 0;
                    }
                    else {
                        sprintf(strLog, "Debug Enabled");
                        strcpy(menu->item[2].str, "Debug OFF");
                        strcpy(menu->item[3].str, "Fix Running Status");
                        strcpy(menu->item[4].str, "Dump Running Status");
                        debug_enabled = 1;
                    }
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    return 0;
                case 2:
                    if (debug_enabled) {
                        fix_running_status();
                    }
                    return 0;
                case 3:
                    if (debug_enabled) {
                        TAP_Print("\n");
//these are crystal palace numbers! - they may be different in the regions ...
                        ret = dump_now_next(0x1044); //bbc1
                        ret = dump_now_next(0x1084); //bbc2
                        ret = dump_now_next(0x2045); //itv1
                        ret = dump_now_next(0x20c0); //ch4
                        ret = dump_now_next(0x3242); //five
                        //ret = dump_now_next(0x41c0); //bbc4
                        //ret = dump_now_next(0x2100); //e4
                    }
                    return 0;
                case 4:
                    close();
                    TAP_Exit();
                    return 0;
                case 5:
                    ShowMenu(&more_menu);
                    return 0;
            }
            break;
        case RKEY_Recall:
            DeleteMenu(menu);
            return param1; // cascades key to other TAP's
        case RKEY_Exit:
            DeleteMenu(menu);
            return 0; // exit without Cascading
        default:
            TAP_Win_Action(&menu->win, param1); // send all other key's to menu-Win
            return 0;
    }
    return param1;
}

int more_menu_handler(type_menu *menu, dword param1)
{
    int selection;
    int num_events;
    int main_tuner, sub_tuner;
    dword heapSize, freeHeapSize, availHeapSize;
    int i,j;
    int ret;
    char str1[0x100];
    int channel_type, channel_number;
    int nibble1, nibble2;
    word mjd;
    byte hour, min, sec;
    word mjd1;
    byte hour1, min1;
    dword utc, local;

    switch (param1) {
        case RKEY_Ok:
            selection = TAP_Win_GetSelection(&menu->win);
            DeleteMenu(menu);
            switch (selection) {
                case 0:
                    TAP_Print("\nfw_max_events = %d", fw_max_events);
                    TAP_Print("\nfw_event_table_size = %x", fw_event_table_size);
                    num_events = exTAP_EPG_GetNumEvents();
                    sprintf(strLog, "Number of Events = %d", num_events);
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
                    return 0;
                case 1:
                    sprintf(strLog, "Deleting all Events");
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_SHORT);
                    num_events = exTAP_EPG_DeleteAllEvents();
                    sprintf(strLog, " = %d", num_events);
                    log_message(LOG_SERIAL, strLog, DELAY_SHORT);
                    return 0;
                case 2:
                    main_tuner = TAP_GetTuner(CHANNEL_Main);
                    sub_tuner = TAP_GetTuner(CHANNEL_Sub);
                    sprintf(strLog, "Main: using tuner %d, PIP: using tuner %d", main_tuner, sub_tuner);
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
                    return 0;
                case 3:
                    TAP_Channel_GetCurrent(&channel_type, &channel_number);
                    ret = exTAP_GetCurrentGenre(channel_type, channel_number, &nibble1, &nibble2);
                    if (ret == 0) {
                        sprintf(strLog, "Current program: genre: [%x %x] %s", nibble1, nibble2, get_content_type_str(nibble1, nibble2));
                        log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
                    }
                    else {
                        TAP_Print("\nexTAP_GetCurrentGenre(), ret=%x", ret);
                    }
                    return 0;
                case 4:
                    return 0;
                case 5:
                    return 0;
                case 6:
                    TAP_GetTime(&mjd, &hour, &min, &sec);
                    sprintf(str1, "LOCAL:     mjd=%04x, %s, %02d:%02d:%02d         ", mjd, mjd_to_string(mjd), hour, min, sec);
                    strcpy(strLog, str1);

                    exTAP_GetUTCTime(&mjd1, &hour1, &min1);
                    utc = (mjd1<<16) + (hour1<<8) + min1;
                    sprintf(str1, "UTC:       mjd=%04x, %s, hour=%02d:%02d       ", mjd1,  mjd_to_string(mjd1), hour1, min1);
                    strcat(strLog, str1);

                    local = exTAP_UTCtoLocalTime(utc);
                    sprintf(str1, "UTC2LOCAL: mjd=%04x, %s, hour=%02d:%02d", local>>16,  mjd_to_string(local>>16), (local>>8)&0xff, local&0xff);
                    strcat(strLog, str1);

                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_MEDIUM);
                    return 0;
                case 7:
                    TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
                    sprintf(strLog,"heapSize=%d        freeHeapSize=%d        availHeapSize=%d", heapSize, freeHeapSize, availHeapSize);
                    log_message(LOG_SCREEN_CONT|LOG_SERIAL, strLog, DELAY_LONG);
                    return 0;
            }
            break;
        case RKEY_Recall:
            DeleteMenu(menu);
            return param1; // cascades key to other TAP's
        case RKEY_Exit:
            DeleteMenu(menu);
            return 0; // exit without Cascading
        default:
            TAP_Win_Action(&menu->win, param1); // send all other key's to menu-Win
            return 0;
    }
    return param1;
}

int key_handler(dword param1)
{
    switch(param1) {
        //only if tsrcommander is not running
        case RKEY_Recall:
            if (!is_tsr_commander_running()) {
                ShowMenu(&main_menu);
                return 0;
            }
            break;
    }
    return param1;
}

int init(void)
{
    dword heapSize, freeHeapSize, availHeapSize;

    main_menu.visible = 0;
    more_menu.visible = 0;

    TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
    sprintf(strLog,"heapSize=%d    freeHeapSize=%d    availHeapSize=%d", heapSize, freeHeapSize, availHeapSize);
    log_message(LOG_SERIAL, strLog, DELAY_SHORT);

    init_crc();
    init_service_id_filter();

//    thirty_seconds = 0;
//    strcpy(main_menu.item[1].str, "Enable Running Status Fix");

    //recheck every 30 seconds
    thirty_seconds = TAP_GetTick() + (30*100);
    strcpy(main_menu.item[1].str, "Disable Running Status Fix");

    debug_enabled = 0;

    TSRCommanderInit(0, TRUE);
    return 0;
}

int close(void)
{
    dword heapSize, freeHeapSize, availHeapSize;

    sprintf(strLog, "Closing  %s", tap_name);
    log_message(LOG_SERIAL|LOG_SCREEN_WAIT, strLog, DELAY_SHORT);

    close_service_id_filter();

    DeleteMenu(&more_menu);
    DeleteMenu(&main_menu);

    TAP_MemInfo(&heapSize, &freeHeapSize, &availHeapSize);
    sprintf(strLog,"heapSize=%d    freeHeapSize=%d    availHeapSize=%d", heapSize, freeHeapSize, availHeapSize);
    log_message(LOG_SERIAL, strLog, DELAY_SHORT);

    if (OpLog) TAP_Hdd_Fclose(OpLog);
    TSRCommanderErase();
    TAP_Print("\nClosing %s", tap_name);
    return 0;
}


//----------------------------------------------------------------------------
void dump_event(type_eventtable *fw_et, int event_index)
{
    int j;
    char event_name[0x100];
    char *description;
    word year;
    byte month, day, weekDay;
    dword utc, local;
    word start_date;
    byte start_hour, start_min;
    word end_date;
    byte end_hour, end_min;


    if (fw_et->char00 & 0xc0) {
//        TAP_Print("\n");
        TAP_Print("[%04x:%04x:%04x]", fw_et->previous, event_index, fw_et->next);
        TAP_Print(" [%04x:%04x:%04x:%04x]", fw_et->original_network_id, fw_et->transport_stream_id, fw_et->service_id, fw_et->event_id);

        utc = (fw_et->start_date << 16) + (fw_et->start_hour<<8) + fw_et->start_min;
        local = exTAP_UTCtoLocalTime(utc);
        start_date = (local>>16)&0xffff;
        start_hour = (local>>8)&0xff;
        start_min = (local>>0)&0xff;
        TAP_ExtractMjd(start_date, &year, &month, &day, &weekDay);

        TAP_Print(" ");
        if (start_date) TAP_Print("[%02d/%02d/%04d", day, month, year);
        TAP_Print(", %02d:%02d", start_hour, start_min);

        utc = (fw_et->end_date << 16) + (fw_et->end_hour<<8) + fw_et->end_min;
        local = exTAP_UTCtoLocalTime(utc);
        end_date = (local>>16)&0xffff;
        end_hour = (local>>8)&0xff;
        end_min = (local>>0)&0xff;
        TAP_ExtractMjd(end_date, &year, &month, &day, &weekDay);

        TAP_Print("-%02d:%02d", end_hour, end_min);
        TAP_Print("]" );

        TAP_Print(" [%x]", fw_et->char00 >> 6);     //valid flag

        TAP_Print(" [%s]", (((fw_et->char00 >> 3) & 7) == 1) ? "  next " :
                           (((fw_et->char00 >> 3) & 7) == 2) ? "shortly" :
                           (((fw_et->char00 >> 3) & 7) == 3) ? "pausing" :
                           (((fw_et->char00 >> 3) & 7) == 4) ? "  now  " :
                                                               "       ");

        TAP_Print(" [%x %x]", fw_et->content_nibble1, fw_et->content_nibble2);

        memcpy(event_name, fw_et->event_name, fw_et->event_name_length);
        event_name[fw_et->event_name_length] = '\0';
        TAP_Print(" %s", event_name);
    }
}


int dump_now_next(int service_id)
{
    type_eventtable *fw_et;
    int event_index;
    int i,j,k;
    dword utc, local;
    word start_date;
    word mjd;
    byte hour, min, sec;
    word mjd1;
    byte hour1, min1;

    TAP_Print("\n");

    //goto the start of the chain
    event_index = 0;
    k = 0;
    while (k < fw_max_events) {
        fw_et = (type_eventtable *)((dword)fw_event_table + (event_index * fw_event_table_size));

        if ((fw_et->char00 & 0xc0) == 0) {
            //find a valid entry
            event_index++;
        }
        else {
            //is this the start
            if (fw_et->previous == 0xffff) {
                break;
            }
            event_index = fw_et->previous;
        }
        k++;
    }
    if (k == fw_max_events) {
        //epg empty
        TAP_Print("\nepg empty");
        return -1;
    }

//    TAP_Print("\nbeginning [%04x:%04x:%04x]", fw_et->previous, event_index, fw_et->next);

    j = event_index;
    k = 0;
    //stop after 8 events
    while(k < 8) {
        fw_et = (type_eventtable *)((dword)fw_event_table + (j * fw_event_table_size));

        //match service_id
        if (fw_et->service_id == service_id) {
            TAP_Print("\n");
            dump_event(fw_et, j);
            k++;
        }
        j = fw_et->next;
        if (j == 0xffff) break;
    }
}



int num_service_filters = 0;
int *running_found;

int init_service_id_filter()
{
    int i;
    int nTvSvc;
    int nRadioSvc;
    TYPE_TapChInfo chInfo;

    TAP_Channel_GetTotalNum(&nTvSvc, &nRadioSvc);
    num_service_filters = nTvSvc + nRadioSvc;

    running_found = malloc(num_service_filters * sizeof(int));
    service_hash = open_hash(2, num_service_filters);

    //create the service list; tv + radio
    for (i=0; i<nTvSvc; i++) {
        TAP_Channel_GetInfo(SVC_TYPE_Tv, i, &chInfo);
        addhash(service_hash, (byte *)&chInfo.svcId, i);
    }
    for (i=0; i<nRadioSvc; i++) {
        TAP_Channel_GetInfo(SVC_TYPE_Radio, i, &chInfo);
        addhash(service_hash, (byte *)&chInfo.svcId, nTvSvc+i);
    }

    return 0;
}

int close_service_id_filter()
{
    free(running_found);
    close_hash(service_hash);
    service_hash = 0;
    return 0;
}

//
//scan epg from the end
//check for multiple running on a channel
//remove the earliest
//
int fix_running_status()
{
    type_eventtable *fw_et;
    int event_index;
    int i,j,k;
    int running_status;
    type_hash_element *hashptr;
    int service_index;
    int previous;


    //goto the end of the chain
    event_index = 0;
    k = 0;
    while (k < fw_max_events) {
        fw_et = (type_eventtable *)((dword)fw_event_table + (event_index * fw_event_table_size));

        //find a valid entry
        if ((fw_et->char00 & 0xc0) == 0) {
            event_index++;
        }
        else {
            //is this the end
            if (fw_et->next == 0xffff) {
                break;
            }
            event_index = fw_et->next;
        }
        k++;
    }
    if (k == fw_max_events) {
        //epg empty
        TAP_Print("\nepg empty");
        return -1;
    }
//    TAP_Print("\nend [%04x:%04x:%04x]", fw_et->previous, event_index, fw_et->next);

    j = event_index;
    k = 0;
    for (i=0; i<num_service_filters; i++) {
        running_found[i] = 0;
    }
    while(k < fw_max_events) {
        fw_et = (type_eventtable *)((dword)fw_event_table + (j * fw_event_table_size));
        previous = fw_et->previous;

//do this for each service id
//use a hash to prevent multiple scanning of the epg tables

        //match the service id
        hashptr = findhash(service_hash, (byte *)&fw_et->service_id);
        if (!hashptr) {
            if (debug_enabled) {
                TAP_Print("\nmiss, service_id=%04x", fw_et->service_id);
            }
        }
        else {
            //hit
            service_index = hashptr->data;
//            TAP_Print("\nhit, service_index=%x", service_index);

            running_status = (fw_et->char00 >> 3) & 7;
            if (running_status == 4) {
                if (!running_found[service_index]) {
                    running_found[service_index] = fw_et->event_id;
                }
                else {
//the firmeware will often end up with duplicate entries of the same program event
//this generally does seem to causes any problems (other than wasting entry space), but if the multiple
//entries have a different running status, it may not be good.  This would be a good place to remove
//all duplicate entries - currecntly it only removes duplicate entries that are also set to 'running'.

                    //if it is a duplicate entry, remove it completely
                    if (running_found[service_index] == fw_et->event_id) {
                        if (debug_enabled) {
                            TAP_Print("\ndelete, ");
                            dump_event(fw_et, j);
                        }
                        exTAP_CallFirmware(fw_event_delete, j, 0,0,0, 0,0,0,0,0,0,0);
                    }
                    else {
                        //clear the running status
                        if (debug_enabled) {
                            TAP_Print("\nclear,  ");
                            dump_event(fw_et, j);
                        }
                        fw_et->char00 &= ~0x38;
                    }
                }
            }
            k++;
        }
        j = previous;
        if (j == 0xffff) break;
    }
    return 0;
}

//----------------------------------------------------------------------------
